#!/bin/python3

import argparse

# Create a parser
parser = argparse.ArgumentParser(description='Patch a M10 firmware for specific frequenc.\nThis is for M10 based on MSP430F233.')

#
# Add arguments
parser.add_argument('--input', type=str, help='Input file', required=True)
parser.add_argument('--read', type=bool, help='Read the frequency value', required=False)
parser.add_argument('--output', type=str, help='Output file', required=False)
parser.add_argument('--frequency', type=int, help='Frequency in Hz', required=False)
parser.add_argument('--verbose', action='store_true', help='Print the details')
args = parser.parse_args()

if args.verbose:
  print(args.input)
  print(args.output)
  print(args.frequency)

def hex_to_freq(hex_value):
  print("hex value : " + hex_value)
  value = int(hex_value, 16)
  print("int value : " + str(value))
  value = str(bin(value))[2:]
  print("binary value : " + value)
  value = value[:-2]
  print("binary value : " + value)
  value = int(value, 2)
  print("int value : " + str(value))
  value = value / 512
  print("freq value : " + str(value))
  return value

# function to convert frequence to hex
def freq_to_hex(freq_base):
  freq=freq_base/1000
  if args.verbose:
    print("Frequency: " + str(freq) + "Hz")
    print("freq * 512 = " + str(freq*512))
    print("arrondu freq * 512 = " + str(int(freq*512)))
    print("binary : " + str(bin(int(freq*512))))
    print("binare + 01 : " + str(bin(int(freq*512)) + "01"))
    print("binary str : " + str(bin(int(freq*512)) + "01").replace("0b",""))

  value = str(hex(int(str(bin(int(freq*512)) + "01").replace("0b",""),2)))[2:]
  print("hex value calculated : 0x" + value)

  # invert the value
  # draft and dirty value flip-flop
  value1="0" + value[0:1]
  value2=value[1:3]
  value3=value[3:5]
  if args.verbose:
    print("value : " + value)
    print ("value1 : " + value1 + " value2 : " + value2 + " value3 : " + value3)
    print("hex value inverted : 0x" + value3 + value2 + value1)
  return value3 + value2 + value1

# check if the input file exists
try:
  with open(args.input, 'rb') as f:
    pass

except FileNotFoundError:
  print(f'Error: {args.input} does not exist')
  exit(1)

if args.read:
  print("")
  print("Reading the frequency value for M10...")
  # print each line of the input file
  with open(args.input, 'rb') as f:
    for line in f:
      if ":10101000" in line.decode('utf-8'):
        if args.verbose:
          print("Found the frequency line")
          print(line.decode('utf-8')[10:15])
        print("Frequency value : " + str(hex_to_freq(line.decode('utf-8')[10:15])))
        exit(0)


print("")
print("Calculating the frequency "+ str(args.frequency) +"Hz value for M10...")

# print each line of the input file
with open(args.input, 'rb') as f:
  for line in f:
    if ":10101000" in line.decode('utf-8'):
      if args.verbose:
        print("Found the frequency line")
        print(line.decode('utf-8')[10:15])
        print("Replacing the frequency value")
      line = line.decode('utf-8')[:10] + freq_to_hex(args.frequency) + line.decode('utf-8')[12:40]
      # Missing the checksum calculation
      print(line)
      # checksum intel hex format
      the_bytes = line[1:].encode('utf-8')
      print(the_bytes)
      a = b'%02X' % (sum(the_bytes) & 0xFF)
